import os, argparse, pathlib, json
from chatchat.base import Base

__platform_config__ = {
    'alibaba': ['api_key'],
    'baidu': ['api_key', 'secret_key'],
    'tencent': ['secret_id', 'secret_key'],
    'xunfei': ['app_id', 'api_key', 'api_secret'],
    'deepseek': ['api_key'],
}

def supported_platforms():
    print(f'Supported platforms:')
    for plat, attrs in __platform_config__.items():
        print(plat)
        for attr in attrs:
            print(f'\t{attr}')

def parse_config(args):
    if args.list:
        supported_platforms()
    elif args.cfgs:
        cfg = args.cfgs.split('.')
        usage = 'Usage: chatchat config platform.key=value'
        if len(cfg) != 2:
            print(usage)
            return

        plat = cfg[0]
        if plat not in __platform_config__:
            print(f'Platform <{plat}> is currently NOT supported!')
            supported_platforms()
            return

        kv = cfg[1].split('=')
        if len(kv) != 2:
            print(usage)
            return
        key, value = kv

        if key not in __platform_config__[plat]:
            print(f'Platform <{plat}> do NOT has secret key <{key}>!\nYou can set the following keys:')
            for key in __platform_config__[plat]:
                print(f'\t{key}')
            return

        util = Base()
        dot_filename = util.secret_file
        dot_content = util.secret_data

        if plat in dot_content:
            dot_content[plat][key] = value
        else:
            dot_content[plat] = {key: value}
        util.write_json(dot_filename, dot_content)

def cli_config(subparser):
    config_parser = subparser.add_parser('config', help='config platform secret key')
    config_parser.add_argument('cfgs', type=str, nargs='?')
    config_parser.add_argument('--list', action='store_true')
    config_parser.set_defaults(parser=parse_config)



from chatchat.base import Base
import httpx

class Completion(Base):
    def __init__(self, model='qwen-turbo', proxy=None, timeout=None):
        super().__init__()

        plat = 'alibaba'
        self.verify_secret_data(plat, ('api_key',))
        self.jdata = self.secret_data[plat]

        # https://dashscope.console.aliyun.com/dashboard?apiKey=all&model=qwen-turbo
        self.model_type = set([
            'qwen-turbo',
            'qwen-plus',
            'qwen-max',
        ])

        if model not in self.model_type:
            raise RuntimeError(f'supported chat type: {list(self.model_type)}')
        self.model = model

        self.api = 'https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation'
        self.client = httpx.Client(proxy=proxy, timeout=timeout)
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.jdata["api_key"]}',
        }

    def send_message(self, messages: list):
        jmsg = {
            'model': self.model,
            "input": {
                "messages": messages,
            },
            "parameters": {
                "result_format": "message"
            }
        }
        r = self.client.post(self.api, headers=self.headers, json=jmsg)
        return r.json()

    def create(self, message):
        jmsg = [{
            "role": "user",
            "content": message,
        }]
        return self.send_message(jmsg)

class Chat(Completion):
    def __init__(self, model='qwen-turbo', history=[], proxy=None, timeout=None):
        super().__init__(model=model, proxy=proxy, timeout=timeout)
        self.history = history

    def chat(self, message):
        self.history.append({
            'role': 'user',
            'content': message,
        })

        r = self.send_message(self.history)
        assistant_output = r['output']['choices'][0]['message']
        self.history.append(assistant_output)

        return r



from chatchat.base import Base
import httpx

class Completion(Base):
    def __init__(self, model='deepseek-chat', proxy=None, timeout=None):
        super().__init__()

        plat = 'deepseek'
        self.verify_secret_data(plat, ('api_key',))
        self.jdata = self.secret_data[plat]

        self.model_type = set([
            'deepseek-chat',
            'deepseek-coder',
        ])

        if model not in self.model_type:
            raise RuntimeError(f'supported chat type: {list(self.model_type)}')
        self.model = model

        self.host = 'https://api.deepseek.com'
        self.model_url = f'{self.host}/models'
        self.chat_url = f'{self.host}/chat/completions'
        self.client = httpx.Client(proxy=proxy, timeout=timeout)
        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.jdata["api_key"]}',
        }

    def model_list(self):
        r = self.client.get(self.model_url, headers=self.headers)
        return r.json()

    def create(self, message, max_tokens=1024, temperature=1.0, top_p=1.0):
        jmsg = {
            'model': self.model,
            'messages': [{
                "role": "user",
                "content": message,
            }],
            'max_tokens': max_tokens,
            'temperature': temperature,
            'top_p': top_p,
        }
        r = self.client.post(self.chat_url, headers=self.headers, json=jmsg)
        return r.json()

class Chat(Completion):
    def __init__(self, model='deepseek-chat', history=[], proxy=None, timeout=None):
        super().__init__(model=model, proxy=proxy, timeout=timeout)
        self.history = history

    def chat(self, message, max_tokens=1024, temperature=1.0, top_p=1.0):
        self.history.append({
            'role': 'user',
            'content': message,
        })

        jmsg = {
            'model': self.model,
            'messages': self.history,
            'max_tokens': max_tokens,
            'temperature': temperature,
            'top_p': top_p,
        }
        r = self.client.post(self.chat_url, headers=self.headers, json=jmsg)
        r = r.json()

        if 'choices' in r:
            assistant_output = r['choices'][0]['message']
            self.history.append(assistant_output)

        return r



from wsgiref.handlers import format_date_time
from urllib.parse import urlencode
from chatchat.base import Base
import base64, hashlib, hmac
from datetime import datetime
from time import mktime
import websocket, json, ssl

class Completion(Base):
    def __init__(self, model='Spark Lite'):
        super().__init__()

        # https://console.xfyun.cn/services/bm2
        # "xunfei": {
        #     "app_id": "x",
        #     "api_secret": "y",
        #     "api_key": "z"
        # }
        plat = 'xunfei'
        self.verify_secret_data(plat, ('api_key', 'api_secret', 'app_id'))
        self.jdata = self.secret_data[plat]

        # https://www.xfyun.cn/doc/spark/Web.html#_1-接口说明
        self.api_list = {
            'Spark3.5 Max': {
                'path': '/v3.5/chat',
                'domain': 'generalv3.5',
            },
            'Spark Pro': {
                'path': '/v3.1/chat',
                'domain': 'generalv3',
            },
            'Spark V2.0': {
                'path': '/v2.1/chat',
                'domain': 'generalv2',
            },
            'Spark Lite': {
                'path': '/v1.1/chat',
                'domain': 'general',
            },
        }

        if model not in self.api_list:
            raise RuntimeError(f'supported chat type: {self.api_list.keys()}')
        self.host = 'spark-api.xf-yun.com'
        self.api = self.api_list[model]
        self.path = self.api['path']
        self.domain = self.api['domain']

        self.update_interval = 150
        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        websocket.enableTrace(False)
        self.answer = ''

    def create_url(self):
        # https://www.xfyun.cn/doc/spark/general_url_authentication.html
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        signature_raw = f'host: {self.host}\ndate: {date}\nGET {self.path} HTTP/1.1'
        signature_sha = hmac.new(
            self.jdata['api_secret'].encode('utf-8'),
            signature_raw.encode('utf-8'),
            digestmod=hashlib.sha256,
        ).digest()
        signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding='utf-8')
        authorization_raw = ', '.join([
            f'api_key="{self.jdata["api_key"]}"',
            'algorithm="hmac-sha256"',
            'headers="host date request-line"',
            f'signature="{signature_sha_base64}"',
        ])
        authorization = base64.b64encode(authorization_raw.encode('utf-8')).decode(encoding='utf-8')
        query = {
            "authorization": authorization,
            "date": date,
            "host": self.host
        }
        url = f'wss://{self.host}{self.path}?{urlencode(query)}'

        return url

    def on_error(self, wsapp, error):
        print(f'Error: {error}')

    def on_close(self, wsapp, close_status_code, close_msg):
        ...

    def on_open(self, wsapp):
        data = json.dumps(wsapp.json)
        wsapp.send(data)

    def on_message(self, wsapp, message):
        data = json.loads(message)
        code = data['header']['code']
        if code != 0:
            print(f'Request Error: {code}, {data}')
            wsapp.close()
        else:
            choices = data["payload"]["choices"]
            status = choices["status"]
            content = choices["text"][0]["content"]
            self.answer += content
            if wsapp.stream: print(content, end='')
            if status == 2:
                wsapp.close()

    def make_message(self, history: list):
        jmsg = {
            "header": {
                "app_id": self.jdata['app_id'],
            },
            "parameter": {
                "chat": {
                    "domain": self.domain,
                }
            },
            "payload": {
                "message": {
                    "text": history
                }
            }
        }
        return jmsg

    def create(self, message, stream=False):
        jmsg = self.make_message([{
            "role": "user", "content": message
        }])

        self.answer = ''
        url = self.create_url()
        ws = websocket.WebSocketApp(
            url, on_message=self.on_message, on_error=self.on_error,
            on_close=self.on_close, on_open=self.on_open,
        )
        ws.json = jmsg
        ws.stream = stream
        ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
        return self.answer

class Chat(Completion):
    def __init__(self, model='Spark Lite', history=[]):
        super().__init__(model=model)
        self.history = history

    def chat(self, message, stream=False):
        self.history.append({
            "role": "user", "content": message
        })
        jmsg = self.make_message(self.history)

        self.answer = ''
        url = self.create_url()
        ws = websocket.WebSocketApp(
            url, on_message=self.on_message, on_error=self.on_error,
            on_close=self.on_close, on_open=self.on_open,
        )
        ws.json = jmsg
        ws.stream = stream
        ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})

        self.history.append({
            "role": "assistant", "content": self.answer,
        })

        return self.answer



from chatchat.base import Base
import httpx, time

class Completion(Base):
    def __init__(self, model='ERNIE-Speed-8K', proxy=None, timeout=None):
        super().__init__()

        # https://console.bce.baidu.com/qianfan/ais/console/applicationConsole/application
        #     {
        #         "baidu": {
        #             "api_key": "x",
        #             "secret_key": "y",
        #             "expires_in": "z",
        #             "access_token": "k"
        #         }
        #     }
        plat = 'baidu'
        self.verify_secret_data(plat, ('api_key', 'secret_key'))
        self.jdata = self.secret_data[plat]
        self.update_interval = 3600

        # https://console.bce.baidu.com/qianfan/ais/console/onlineService
        self.api_list = {
            'ERNIE-Speed-8K': 'ernie_speed',
            'ERNIE-Speed-128K': 'ernie-speed-128k',
            'ERNIE-Speed-AppBuilder': 'ai_apaas',
            'ERNIE-Lite-8K': 'ernie-lite-8k',
            'ERNIE-Lite-8K-0922': 'eb-instant',
            'ERNIE-Bot-turbo-0922': 'eb-instant',
            'ERNIE-Tiny-8K': 'ernie-tiny-8k',
            'Yi-34B-Chat': 'yi_34b_chat',
        }

        if model not in self.api_list:
            raise RuntimeError(f'supported chat type: {self.api_list.keys()}')
        self.api = 'https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/' + self.api_list[model]
        self.client = httpx.Client(proxy=proxy, timeout=timeout)
        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        self.update_access_token()

    def update_access_token(self):
        if 'expires_in' not in self.jdata or not self.jdata['expires_in'] \
            or self.jdata['expires_in'] < time.time() + self.update_interval:
            # https://cloud.baidu.com/doc/WENXINWORKSHOP/s/Ilkkrb0i5
            url = "https://aip.baidubce.com/oauth/2.0/token"
            params = {
                "grant_type": "client_credentials",
                "client_id": self.jdata['api_key'],
                "client_secret": self.jdata['secret_key'],
            }
            # {
            #     "refresh_token":"a",
            #     "expires_in": b,
            #     "session_key": "c",
            #     "access_token": "d",
            #     "scope": "e",
            #     "session_secret": "f"
            # }
            cur_time = time.time()
            r = self.client.post(url, headers=self.headers, params=params).json()
            self.jdata['access_token'] = r['access_token']
            self.jdata['expires_in'] = cur_time + float(r['expires_in'])
            jdata = self.load_json(self.jfile)
            jdata.update({'baidu': self.jdata})
            self.write_json(self.jfile, jdata)

    def get_access_token(self):
        self.update_access_token()
        return self.jdata['access_token']

    def send_messages(self, messages: list):
        jmsg = {"messages": messages}
        url = f'{self.api}?access_token={self.get_access_token()}'
        r = self.client.post(url, headers=self.headers, json=jmsg)
        return r.json()

    def create(self, message):
        messages = [
            {
                "role": "user",
                "content": message,
            }
        ]
        return self.send_messages(messages)

class Chat(Completion):
    def __init__(self, model='ERNIE-Speed-8K', history=[], proxy=None, timeout=None):
        super().__init__(model=model, proxy=proxy, timeout=timeout)
        self.history = history

    def chat(self, message):
        self.history.append({
            "role": "user",
            "content": message,
        })
        r = self.send_messages(self.history)
        if 'result' in r:
            self.history.append({
                "role": "assistant",
                "content": r['result']
            })

        return r



import json, pathlib, os

class Base():
    def __init__(self):
        dot_secret_file = os.path.join(
            str(pathlib.Path.home()),
            '.chatchat.json',
        )
        if not os.path.exists(dot_secret_file):
            self.write_json(dot_secret_file, {})

        self.secret_data = self.load_json(dot_secret_file)
        self.secret_file = dot_secret_file

    def verify_secret_data(self, plat, keys):
        has_plat = plat in self.secret_data
        has_key = True
        if has_plat:
            plat_data = self.secret_data[plat]
            for key in keys:
                has_key = has_key and key in plat_data
        if not (has_plat and has_key):
            print('\nYou must set:')
            for key in keys:
                print(f'\tchatchat config {plat}.{key}=YOUR_{key.upper()}')
            exit(-1)

    def load_json(self, jfile):
        with open(jfile) as jf:
            data = json.load(jf)
        return data

    def write_json(self, jfile, jdata):
        with open(jfile, 'w+') as jd:
            json.dump(jdata, jd, indent=4)